#!/bin/bash

function usage {
cat >&2 << __EOF__ 
Usage: $0 [-vfpsmutb] [-D DPI] [-d depth] [-F format] pdffile outdir
Convert pdffile - a pdf of scanned facing pages - to a set of images in outdir,
each with only one page.
OPTIONS:
	-v: Be verbose.
	-d depth: Use given depth. Default is 1 for black and white, 8 for colour/gray.
		Unpaper can only handle up to 8.
	-D dpi: Use given DPI. Colour scans on library scanners are usually 300, bw 400.
	-e extension: use the output format signified by the extension. Default is png
	-b: Optimize for BeBook One.
	-1: each physical page contains one logical page
	-2: each physical page contains two logical pages
		(neither specified: try to figure it out)
__EOF__
}


## Setup and utilities ##
source $(dirname $0)/my_seq.sh

## Process flags ##
convertflags=
unpaperflags=
depthset=
depth=
dpi=
skipmask=
verbose=
extension="png"
bebook=
logperphys=
while getopts 'vd:D:e:b12' OPTION
do
	case $OPTION in
	v)	verbose=1
		convertflags="$convertflags -verbose"
		unpaperflags="$unpaperflags -v --time"
		;;
	d)	depth="$OPTARG"
		;;
	D)	dpi=$OPTARG
		;;
	e)	extension="$OPTARG"
		;;
	b)	bebook=1
		;;
	1)	logperphys=1
		;;
	2)	logperphys=2
		;;
	?)	usage
		exit 2
		;;
	esac
done
shift $(($OPTIND - 1))

# check we have an input and output!
if [[ $# != 2 ]]; then
	echo "Wrong number of parameters (2 required, $# given: [$@])" >&2
	usage
	exit 2
fi

scanfile=$1
outdir=$2

filedir=$(dirname $1)
base=$(basename $1 .pdf)

# make the output dir
mkdir -p $outdir


# guess at DPI, unless it's been set
# the scans do not seem to accurately reflect their resolution.
if [[ "$dpi" = "" ]]; then
	# %k gives the number of colours/shades in the image.
	# 2 for b/w, and the scanner seems to default to 400 dpi for that
	# 256 for greyscale, which has to be selected manually (so I'm assuming 300 for no good reason)
	# 31564 for colour, which defaults to 300
	if [[ $( convert $scanfile[0] -format "%k" info: || exit 1 ) == 2 ]]; then
		dpi=400;
	else
		dpi=300
	fi
	if [[ $verbose == 1 ]]; then
		echo "DPI guessed at $dpi."
	fi
fi

# guess at depth, unless it's been set
if [[ "$depth" = "" ]]; then
	if [[ $( convert $scanfile[0] -format "%k" info: || exit 1 ) == 2 ]]; then
		depth=1;
	else
		depth=8
	fi
	if [[ $verbose == 1 ]]; then
		echo "Depth guessed at $depth."
	fi
fi

# figure out the number of pages
dscname=$outdir/${base}.dsc
pdf2dsc $scanfile $dscname || exit 1
pages=$(awk '$1 ~ "%%Pages" {print $2}' $dscname)
echo "Got $pages page(s)."
rm $dscname

# process pages 1 by 1 to avoid convert gobbling all the memory
for scanpgnum in `$my_seq 1 $pages`; do
	echo "Processing page $scanpgnum."
	
	scanpgnum=$(printf '%03d' $scanpgnum)
	
	# convert from pdf
	scanpg=$outdir/scanpg-${scanpgnum}.png
	if [ ! -e $scanpg ]; then
		convert $convertflags -depth $depth -density $dpi $scanfile[$(expr $scanpgnum - 1)] \
			$scanpg || exit 1
	fi;
	
	# preprocess it!
	cleanscanpg=$outdir/scanpg-clean-${scanpgnum}.pnm
	if [ ! -e $cleanscanpg ]; then
		# create mask: 
		# ... downscale, blur,
		convert $convertflags -resize 10% -depth 8 -blur 10 -median 2 $scanpg $outdir/scanpg-mask-${scanpgnum}.png ||exit 1

		# ... get crop co-ords. They're off by ~2 as I don't know how to
		# properly correct for the border.
		cropcords=$(convert -border 1x1 -bordercolor '#000' -resize 1000% -trim -fuzz 90% -format "%wx%h%O" $outdir/scanpg-mask-${scanpgnum}.png info: || exit 1)
		
		# ... crop and despeckle? the final pre-prepared image
		convert $convertflags -crop $cropcords $scanpg $cleanscanpg || exit 1
	elif [[ $skipmask ]]; then
		cp $origfile $preppnm
	fi;
	
	# check it hasn't mostly disappeared - e.g. if the scan was all black
	# (e.g. forgot to put the book down when you first hit scan)
	if [[ $(convert  $cleanscanpg -format '%[fx:s.w*s.h>1000]' info:) = "0" ]]; then
		[[ $verbose == 1 ]] && echo "Discarding pg ${scanpgnum}: not enough remains after masking."
		continue; 
	fi;
	
	#unpaper it
	physpgbase=$outdir/physpg-${scanpgnum}
	if [ ! -e ${physpgbase}-1.pnm ] || [ ! -e ${physpgbase}-2.pnm ]; then
		unpaper $unpaperflags --layout double --overwrite -ni 10 -op 2 $cleanscanpg ${physpgbase}-%01d.pnm || exit 1
	fi;
	
	for physpgnum in $($my_seq 1 2); do
		echo Processing physical page ${physpgnum}.
		
		physpg=${physpgbase}-${physpgnum}.pnm #$physpgbase has $outdir/ already
		
		cleanphyspg=$outdir/physpg-clean-${scanpgnum}-${physpgnum}.pnm
		
		if [ ! -e $cleanphyspg ]; then
			# remask and retrim. FIXME: may throw off 2page detection
			convert $convertflags -resize 10% -depth 8 -gamma 0.01 -median 2 $physpg $outdir/physpg-mask-${scanpgnum}-${physpgnum}.png ||exit 1

			# Trim #-border 1x1 -bordercolor '#fff' -trim -fuzz 30% 
			cropcords=$(convert -trim -fuzz 90%\
			 -resize 1000% -format "%wx%h%O" $outdir/physpg-mask-${scanpgnum}-${physpgnum}.png info: || exit 1)
		
			# ... crop and despeckle? the final pre-prepared image
			convert $convertflags -crop $cropcords $physpg $cleanphyspg || exit 1
		fi;
		
		#detect if the page is 2-up

		if [[ $logperphys == 2 ]] || ( [[ $logperphys != 1 ]] && $(dirname $0)/detect2pages.sh ${cleanphyspg} ${scanpgnum} ${physpgnum} ); then
			if [[ $verbose == 1 ]]; then
				echo "Resplitting physical page ${physpgnum}."
			fi
			unpaper $unpaperflags --pre-rotate 90 --layout double --overwrite \
			 -op 2 --no-blackfilter --no-grayfilter --no-noisefilter \
			 -no-blurfilter $cleanphyspg $outdir/logpg-${scanpgnum}-${physpgnum}-%01d.pnm || exit 1
		else
			cp $cleanphyspg $outdir/logpg-${scanpgnum}-${physpgnum}-1.pnm
		fi;

	
		#final convert and clean w/ bebook optimisation
		if [[ $bebook ]]; then
			convert $convertflags -colorspace Gray -median 1 $outdir/logpg-${scanpgnum}-${physpgnum}-?.pnm -trim -fuzz 80% -resize 1200x1600 $outdir/final-${scanpgnum}-${physpgnum}-%01d.${extension} || exit 1
		else
			convert $convertflags $outdir/logpg-${scanpgnum}-${physpgnum}-?.pnm $outdir/final-${scanpgnum}-${physpgnum}-%01d.${extension} || exit 1
		fi
	
	done;

done

mkdir -p $outdir/pages
mv $outdir/final-*.${extension} $outdir/pages
	
